/*---[ parse.c ]------------------------------------------------------
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * The text file parse functions.
 * TODO: Maybe rewrite using GStrings and properly modular functions.
 *
 * Based on firestarter code by Tomas Junnonen (majix@sci.fi)
 *--------------------------------------------------------------------*/

#include "razorback.h"
#include "parse.h"

/* [ get_text_between ]
 * Give a string and two text markers in the string the function returns the
 * text between the markers.
 */
static gchar *
get_text_between (gchar *string, gchar *marker1, gchar *marker2)
{
	gchar *text, *marker3;
	gint i = strlen (marker1);

	text = g_strdup (string);
	text = strstr (text, marker1);
	text += i*(sizeof (gchar));
	marker3 = strstr (text, marker2);
	text = g_strndup (text, marker3-text);

	return (text);
}

/* [ parse_hit_log ]
 * Parses *line. Extracts port, remoteip, service, time and protocol
 */
int
parse_hit_log (char *line)
{
	gchar *buf;
	gchar *buf2,*buf3;
	gchar *data[4] = {"source", "category", "description", "time"};
	gchar *proto;
	static gchar *time, *port, *ip;
	static gint datakey = 1;

	data[3] = g_strndup (line, 12);

	if ( (buf = (char *)strstr (line, "snort")) == NULL )
		return FALSE;

	buf = strstr (buf, ":");
	if (buf == NULL)
		return FALSE;

	data[0] = get_text_between (buf, ":", ":");

	buf2 = strstr (buf, "->");
	if(buf2 == NULL)
	{
		// Must a descriptive event. Grab the source IP first.
		buf2=strstr(buf,"from");

		if(buf2 == NULL)
		{
			// Hmm.. no from. Make it up.
			data[1]="Unknown";
		} else {
			// Grab the data between the spaces.
			buf2=get_text_between(buf2," "," ");

			// Trim off the tail colon if it is there.
			buf3 = buf2;
			buf3 += (strlen(buf3) - 1) * sizeof(gchar);
			if(!strncmp(buf3,":",1))
			{
				// Get rid of that colon.
				data[1]=g_strndup(buf2,(strlen(buf2)-1));
			} else {
				data[1]=buf2;
			}
		}
	} else {
		buf=strstr(buf," ");
		buf=strstr(buf,": ");

		buf=get_text_between(buf," "," ");

		buf2=strstr(buf,":");
		if(buf2)
		{
			buf=g_strndup(buf,(strlen(buf)-strlen(buf2)));
		}
		data[1]=buf;
	}

	buf=(char *)strstr (line, "snort");
	buf=strstr(buf," ");
	buf=strstr(buf,": ");
	buf += (2 * sizeof(gchar));
	data[2]=buf;

	/* Reset datakey if the clist is empty */
	if (!gtk_clist_get_text (GTK_CLIST (hitlogclist), 0, 0, NULL))
	datakey = 1;

	/* Remove identical lines */
	if (gnome_config_get_bool ("/razorback/Program/removedups=TRUE")) {
		if ((time != NULL) && (!g_strcasecmp (time, data[3]))) {
			if (!g_strcasecmp (port, data[0]))
				if (!g_strcasecmp (ip, data[1]))
					return FALSE;
		}
		time = data[3];
		port = data[0];
		ip = data[1];
	}

	gtk_clist_append (GTK_CLIST (hitlogclist), data);

	gtk_object_set_data (GTK_OBJECT (hitlogclist),
			     g_strdup_printf ("%d", datakey), proto);
	gtk_object_set_data (GTK_OBJECT (hitlogclist),
			     "keys", GINT_TO_POINTER (datakey));
	++datakey;

	return TRUE;
}
